@@ -58,6 +58,8 @@ extern char **environ;
 #define SERVER_RUNNING 252      /* Server running and ready for connections */
 #define SERVER_ABORT 253        /* Server failed badly. Suspending display. */
 
+#define MAX_LOGS 5
+
 struct GdmServerPrivate
 {
         char    *command;
@@ -319,33 +321,30 @@ gdm_server_resolve_command_line (GdmServer  *server,
 }
 
 static void
-rotate_logs (GdmServer *server)
+rotate_logs (const char *path,
+             guint       n_copies)
 {
-        int   n_copies;
-        int   i;
-        char *filename;
-        char *path;
-
-        n_copies = 5;
-
-        filename = g_strdup_printf ("%s.log", server->priv->display_name);
-        path = g_build_filename (server->priv->log_dir, filename, NULL);
-        g_free (filename);
+        int i;
 
         for (i = n_copies - 1; i > 0; i--) {
                 char *name_n;
                 char *name_n1;
 
                 name_n = g_strdup_printf ("%s.%d", path, i);
-                name_n1 = g_strdup_printf ("%s.%d", path, i - 1);
-                g_unlink (name_n);
-                g_rename (name_n1, name_n);
+                if (i > 1) {
+                        name_n1 = g_strdup_printf ("%s.%d", path, i - 1);
+                } else {
+                        name_n1 = g_strdup (path);
+                }
+
+                VE_IGNORE_EINTR (g_unlink (name_n));
+                VE_IGNORE_EINTR (g_rename (name_n1, name_n));
 
                 g_free (name_n1);
                 g_free (name_n);
         }
 
-        g_unlink (path);
+        VE_IGNORE_EINTR (g_unlink (path));
 }
 
 static void
@@ -403,22 +402,26 @@ change_user (GdmServer *server)
 static void
 server_child_setup (GdmServer *server)
 {
-        char            *logfile;
         int              logfd;
         struct sigaction ign_signal;
         sigset_t         mask;
-        char            *temp;
+        char            *log_file;
+        char            *log_path;
+
+        log_file = g_strdup_printf ("%s.log", server->priv->display_name);
+        log_path = g_build_filename (server->priv->log_dir, log_file, NULL);
+        g_free (log_file);
 
         /* Rotate the X server logs */
-        rotate_logs (server);
+        rotate_logs (log_path, MAX_LOGS);
 
         /* Log all output from spawned programs to a file */
-        temp = g_strconcat (server->priv->display_name, ".log", NULL);
-        logfile = g_build_filename (server->priv->log_dir, temp, NULL);
-        g_debug ("GdmServer: Opening logfile for server %s", logfile);
+        g_debug ("GdmServer: Opening logfile for server %s", log_path);
 
-        VE_IGNORE_EINTR (g_unlink (logfile));
-        VE_IGNORE_EINTR (logfd = open (logfile, O_CREAT|O_TRUNC|O_WRONLY|O_EXCL, 0644));
+        VE_IGNORE_EINTR (g_unlink (log_path));
+        VE_IGNORE_EINTR (logfd = open (log_path, O_CREAT|O_APPEND|O_TRUNC|O_WRONLY|O_EXCL, 0644));
+
+        g_free (log_path);
 
         if (logfd != -1) {
                 VE_IGNORE_EINTR (dup2 (logfd, 1));
@@ -678,32 +681,13 @@ gdm_server_start (GdmServer *server)
         return res;
 }
 
-static int
-wait_on_child (int pid)
-{
-        int status;
-
- wait_again:
-        if (waitpid (pid, &status, 0) < 0) {
-                if (errno == EINTR) {
-                        goto wait_again;
-                } else if (errno == ECHILD) {
-                        ; /* do nothing, child already reaped */
-                } else {
-                        g_debug ("GdmServer: waitpid () should not fail");
-                }
-        }
-
-        return status;
-}
-
 static void
 server_died (GdmServer *server)
 {
         int exit_status;
 
         g_debug ("GdmServer: Waiting on process %d", server->priv->pid);
-        exit_status = wait_on_child (server->priv->pid);
+        exit_status = gdm_wait_on_pid (server->priv->pid);
 
         if (WIFEXITED (exit_status) && (WEXITSTATUS (exit_status) != 0)) {
                 g_debug ("GdmServer: Wait on child process failed");
@@ -726,6 +710,8 @@ server_died (GdmServer *server)
 gboolean
 gdm_server_stop (GdmServer *server)
 {
+        int res;
+
         if (server->priv->pid <= 1) {
                 return TRUE;
         }
@@ -738,8 +724,11 @@ gdm_server_stop (GdmServer *server)
 
         g_debug ("GdmServer: Stopping server");
 
-        gdm_signal_pid (server->priv->pid, SIGTERM);
-        server_died (server);
+        res = gdm_signal_pid (server->priv->pid, SIGTERM);
+        if (res < 0) {
+        } else {
+                server_died (server);
+        }
 
         return TRUE;
 }
@@ -769,6 +758,13 @@ _gdm_server_set_user_name (GdmServer  *server,
         server->priv->user_name = g_strdup (name);
 }
 
+static void
+_gdm_server_set_disable_tcp (GdmServer  *server,
+                             gboolean    disabled)
+{
+        server->priv->disable_tcp = disabled;
+}
+
 static void
 gdm_server_set_property (GObject      *object,
                          guint         prop_id,
@@ -789,6 +785,9 @@ gdm_server_set_property (GObject      *object,
         case PROP_USER_NAME:
                 _gdm_server_set_user_name (self, g_value_get_string (value));
                 break;
+        case PROP_DISABLE_TCP:
+                _gdm_server_set_disable_tcp (self, g_value_get_boolean (value));
+                break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
                 break;
@@ -819,6 +818,9 @@ gdm_server_get_property (GObject    *object,
         case PROP_USER_NAME:
                 g_value_set_string (value, self->priv->user_name);
                 break;
+        case PROP_DISABLE_TCP:
+                g_value_set_boolean (value, self->priv->disable_tcp);
+                break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
                 break;
@@ -912,6 +914,13 @@ gdm_server_class_init (GdmServerClass *klass)
                                                               "user name",
                                                               NULL,
                                                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+        g_object_class_install_property (object_class,
+                                         PROP_DISABLE_TCP,
+                                         g_param_spec_boolean ("disable-tcp",
+                                                               NULL,
+                                                               NULL,
+                                                               TRUE,
+                                                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
 }
 
